/* $Id: cli.c,v 1.7 2004/05/10 01:13:10 playfair Exp $
**
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License as
** published by the Free Software Foundation; either version 2 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
** General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
** 02111-1307, USA.
*/

#include <stdarg.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/stat.h>

#include "config.h"
#include "hymn.h"
#include "filename.h"

#define OUTPUT_FILEXT "m4a"

int errmsg(void *udata, int err, ...) {
    va_list argp;
    va_start(argp, err);
    vfprintf(stderr, ERROR_MESSAGES[err], argp);
    va_end(argp);
    fprintf(stderr, "\n");

    return err;
}

void logmsg(void *udata, int log_lvl, int msg_lvl, char *str, ...) {
    if (log_lvl >= msg_lvl) {
        va_list argp;
        fprintf(stderr, "hymn(%d): ", msg_lvl);
        va_start(argp, str);
        vfprintf(stderr, str, argp);
        va_end(argp);
        fprintf(stderr, "\n");
    }
}

void status(void *udata, char *status) {
    fprintf(stderr, "%s", status);
}

static const char* MSG_VERSION =
    "%s version " VERSION ".\n"
    "Copyright 2004.\n"
    "\n"
    "This is free software; see the source for copying conditions.\n"
    "There is NO warranty; not even for MERCHANTABILITY or FITNESS FOR A\n"
    "PARTICULAR PURPOSE.\n"
    "\n"
    "Report bugs to playfair@mac.hush.com.\n"
    ;
void version(char *prog) {
    printf(MSG_VERSION, prog);
}

static const char* MSG_USAGE =
    "USAGE:\n"
    "  %s [-v] [-l n] [-x ext] <file1> [ [file2] ... [fileN] ] [destdir]\n"
    "\t-l n      set logging level to 'n' (%d-%d, default=%d)\n"
    "\t             %d  supress log messages\n"
    "\t             %d  log warnings\n"
    "\t             %d  log informational messages\n"
    "\t             %d  log debug information\n"
    "\t             %d  log program flow\n"
    "\t             %d  log everything\n"
    "\t-x ext    use 'ext' as the file extension for output files\n"
    "\t-v        display %s version information\n"
    "  %s [-h]\n"
    "\t-h        this help message\n"
    ;
int usage(char *prog) {
    fprintf(stderr, MSG_USAGE, prog,
            LOG_OFF, LOG_SPEW, LOG_WARN,
            LOG_OFF, LOG_WARN, LOG_INFO, LOG_DEBUG, LOG_TRACE, LOG_SPEW,
            prog, prog);
}

int main(int argc, char *argv[]) {
    hymn_callback_t *hcb = 0;
    int err, opt, log_lvl = LOG_WARN, fileidx, numfiles;
    bool dispvers = false;
    char *prog = 0, *destdir = 0, *ext = OUTPUT_FILEXT;

    prog = argv[0];
    if (argc > 1) {
        while ((opt = getopt(argc, argv, "?d:hvx:")) != -1) {
            switch(opt) {
            case 'l':
                log_lvl = atoi(optarg);
                break;
            case 'v':
                dispvers = true;
                break;
            case 'x':
                ext = optarg;
                break;
            case 'h':
            case '?':
            default:
                return usage(prog);
            }
        }
        argc -= optind;
        argv += optind;

        /* display version if asked */
        if (dispvers) {
            version(prog);
        }

        /* make sure they passed at least one file */
        if (argc < 1) {
            /* they may have just wanted version info */
            if (dispvers) {
                return SUCCESS;
            } else {
                return usage(prog);
            }
        }

        /* see if the last argument was a directory */
        struct stat st;
        stat(argv[argc-1], &st);
        if ( (st.st_mode & S_IFMT) == S_IFDIR ) {
            destdir = argv[argc-1];
            numfiles = argc - 1;
            /* trim any trailing file separator */
            if (destdir[strlen(destdir)-1] == FILE_SEP)
                destdir[strlen(destdir)-1] = 0;
        } else {
            destdir = 0;
            numfiles = argc;
        }

        for (fileidx=0; fileidx<numfiles; ++fileidx) {
            char *infile = 0, *outfile = 0;
            filename_t *fn = 0;

            infile = argv[fileidx];
            fn = malloc(sizeof(filename_t));
            parse_filename(infile, &fn);
            outfile = get_outfile(fn, destdir, ext);

            hcb = malloc(sizeof(hymn_callback_t));
            hcb->logmsg = logmsg;
            hcb->errmsg = errmsg;
            hcb->status = status;
            hcb->user_data = 0;
            hcb->log_lvl = log_lvl;

            err = convert(hcb, infile, outfile);

            if (fn) free(fn);
            if (outfile) free(outfile);
            if (hcb) free(hcb);
            if (err) return err;
        }
    } else {
        return usage(prog);
    }

    return SUCCESS;
}

/* EOF */
